local _G = getfenv(0)
local addon, L = ...;

local AWI_lineid = 0 -- if you have whispers for more than one chat tab, check this to only fire once.
local ipairs = ipairs
local color1 = "|cff3399ff"
local color2 = "|cff1eff00"
local keywords = {L["keyword1"]}
if L["keyword2"] ~= "" then table.insert(keywords,L["keyword2"]) end
if L["keyword3"] ~= "" then table.insert(keywords,L["keyword3"]) end
if L["keyword4"] ~= "" then table.insert(keywords,L["keyword4"]) end
if L["keyword5"] ~= "" then table.insert(keywords,L["keyword5"]) end
if L["keyword6"] ~= "" then table.insert(keywords,L["keyword6"]) end
if L["keyword7"] ~= "" then table.insert(keywords,L["keyword7"]) end
if L["keyword8"] ~= "" then table.insert(keywords,L["keyword8"]) end
local keywordlist = ""
local AWI_Loaded = false

-- initialize for savedvariables
_G.AWI_DB = {
   Color1 = color1,
   Color2 = color2,
   Keywords = keywords,
   Version = GetAddOnMetadata(addon, "X-Version") ,
   GlobalSave = false,
   AllInvite = false,
   AutoInvite = false,
   BNet = false,
   Debug = false,
   ExactWhispers = true,
   FriendsInvite = true,
   HideWhispers = false,
   InstanceCheck = false,
   RaidConvert = false,
   RaidInviteDelay = 0.1,
   Status = true,   
}

-- initialize for savedvariables per character
_G.AWI_DBPC = {
   AllInvite = false,
   AutoInvite = false,
   BNet = false,
   Debug = false,
   ExactWhispers = true,
   FriendsInvite = true,
   HideWhispers = false,
   InstanceCheck = false,
   RaidConvert = false,
   RaidInviteDelay = 0.1,
   Status = true,
}

local function LoadLocal()
  _G.AWI_DBPC.AllInvite = _G.AWI_DB.AllInvite
  _G.AWI_DBPC.AutoInvite = _G.AWI_DB.AutoInvite
  _G.AWI_DBPC.BNet = _G.AWI_DB.BNet
  _G.AWI_DBPC.Debug = _G.AWI_DB.Debug  
  _G.AWI_DBPC.ExactWhispers = _G.AWI_DB.ExactWhispers
  _G.AWI_DBPC.FriendsInvite = _G.AWI_DB.FriendsInvite
  _G.AWI_DBPC.HideWhispers = _G.AWI_DB.HideWhispers
  _G.AWI_DBPC.InstanceCheck = _G.AWI_DB.InstanceCheck
  _G.AWI_DBPC.RaidConvert = _G.AWI_DB.RaidConvert
  _G.AWI_DBPC.RaidInviteDelay = _G.AWI_DB.RaidInviteDelay        
  _G.AWI_DBPC.Status = _G.AWI_DB.Status    
end

local function myprint(msg)
  print(_G.AWI_DB.Color1..addon..": ".._G.AWI_DB.Color2..msg)
end

local isRetail = (_G.WOW_PROJECT_ID == _G.WOW_PROJECT_MAINLINE)

local function LoadGlobal()
  if _G.AWI_DB.GlobalSave then
    _G.AWI_DB.AllInvite = _G.AWI_DBPC.AllInvite
    _G.AWI_DB.AutoInvite = _G.AWI_DBPC.AutoInvite
    _G.AWI_DB.BNet = _G.AWI_DBPC.BNet
    _G.AWI_DB.Debug = _G.AWI_DBPC.Debug  
    _G.AWI_DB.ExactWhispers = _G.AWI_DBPC.ExactWhispers
    _G.AWI_DB.FriendsInvite = _G.AWI_DBPC.FriendsInvite
    _G.AWI_DB.HideWhispers = _G.AWI_DBPC.HideWhispers
    _G.AWI_DB.InstanceCheck = _G.AWI_DBPC.InstanceCheck
    _G.AWI_DB.RaidConvert = _G.AWI_DBPC.RaidConvert
    _G.AWI_DB.RaidInviteDelay = _G.AWI_DBPC.RaidInviteDelay        
    _G.AWI_DB.Status = _G.AWI_DBPC.Status    
  end
end

local function isKey(matchword)
  for _, keyword in ipairs(_G.AWI_DB.Keywords) do
    if matchword == keyword then 
      return true 
    end
  end
  return false
end

local function isKeyinText(matchword)
  for _, keyword in ipairs(_G.AWI_DB.Keywords) do
    if _G.AWI_DBPC.ExactWhispers then
      if matchword == keyword then 
        return true 
      end
    else
      if string.find(matchword, keyword) then 
        return true 
      end
    end
  end
  return false
end

local function getKeys()
  keywordlist = ""
  local counter = 0
	for _, keyword in ipairs(_G.AWI_DB.Keywords) do
	  counter = counter +1
    if counter == #_G.AWI_DB.Keywords then
      keywordlist = keywordlist..keyword
    else
      keywordlist = keywordlist..keyword..", "
    end
  end
  return keywordlist
end

local function addKeyword(word)
  word = strtrim(word)
  if word ~= "" then
    word = word:lower()
    if #_G.AWI_DB.Keywords <= 7 then
      if not isKey(word) then
        myprint(string.format(L["KeyAdd"],_G.AWI_DB.Color1..word))
        table.insert(_G.AWI_DB.Keywords,word)
        keywordlist = _G.AWI_DB.Color1..getKeys().._G.AWI_DB.Color2
        myprint(string.format(L["Keys"],keywordlist))
      else
        myprint(L["KeyFound"])
      end
    else
      myprint(L["KeyFull"])
    end
  end
end

local function delKeyword(word)
  if word ~= "" then
    word = strtrim(word)
    word = word:lower()
    if #_G.AWI_DB.Keywords >= 2 then
      if isKey(word) then
        for i, keyword in ipairs(_G.AWI_DB.Keywords) do
          if word == keyword then
            myprint(string.format(L["KeyRemoved"],_G.AWI_DB.Color1))
            table.remove(_G.AWI_DB.Keywords,i)
            keywordlist = _G.AWI_DB.Color1..getKeys().._G.AWI_DB.Color2
            myprint(string.format(L["Keys"],keywordlist))
          end
        end
      else
        myprint(string.format(L["KeyNotFound"],_G.AWI_DB.Color1..word))
      end
    else
      myprint(L["KeyLast"])
    end
  end
end

local function isInGuild(sender)
  if not IsInGuild() then 
    if _G.AWI_DBPC.Debug then 
      myprint(" you are not in a guild") 
    end
    return false 
  else
		if isRetail then
			C_GuildInfo.GuildRoster()
		else
			GuildRoster()
		end
  end
  if string.find(sender, '-') then 
	  sender = strsub(sender, 1, string.find(sender, "-")-1) -- remove realm
	end
 	if _G.AWI_DBPC.Debug then 
 	  myprint(sender.." whisperer") 
 	end
	for i=1, GetNumGuildMembers() do 
		local name, _, _, _, _, _, _, _, isOnline, _, _, _, _, isMobile = GetGuildRosterInfo(i)
		name = Ambiguate(name, "none")
		if isOnline and not isMobile then
		  if name == sender then
        if _G.AWI_DBPC.Debug then 
          myprint(sender.." is in my guild") 
        end
  		  return true
		  end		
		end
	end
  if _G.AWI_DBPC.Debug then 
    myprint(sender.." not in my guild") 
  end
  return false
end

local function isFriend(sender)
  local numFriends = C_FriendList.GetNumFriends()
  if numFriends > 0 then
    if string.find(sender, '-') then 
      sender = strsub(sender, 1, string.find(sender, "-")-1) -- remove realm
    end
    for i = 1, numFriends do
      local f =	C_FriendList.GetFriendInfoByIndex(i) -- name, level, class, area, connected, status, note
      if f.name == sender then
        if _G.AWI_DBPC.Debug then 
          myprint(sender.." is in friends list") 
        end
        return true
      end
    end
  end
  if _G.AWI_DBPC.Debug then 
    myprint(sender.." not in friends list") 
  end
  return false
end

local function isAuthorized(sender, isbnetwhisper)
  if _G.AWI_DBPC.Debug then 
    myprint(" checking authorization for: "..sender) 
  end
  if _G.AWI_DBPC.AllInvite then
    if _G.AWI_DBPC.Debug then 
      myprint(" all are welcome") 
    end
    return true 
  end
  if isInGuild(sender) then return true end -- check if in guild always authorized
  if isbnetwhisper and _G.AWI_DBPC.FriendsInvite then return true end -- bnet whisper is a friend
  if _G.AWI_DBPC.FriendsInvite then return isFriend(sender) end -- is in friends list
  if _G.AWI_DBPC.Debug then 
    myprint(" not authorized") 
  end
  return false
end

local function CheckGroupMembers(sender)
  for i = 1, GetNumGroupMembers() do	
	  if IsInRaid() then
 		  local partyMemberName = UnitName("raid"..i, false)
   	  if partyMemberName ~= nil then
        if not partyMemberName:find("-" .. GetRealmName()) then
     	    partyMemberName = partyMemberName.."-"..GetRealmName()
        end
        if partyMemberName == sender then
   	  	  SendChatMessage(L["IsinRaid"], "WHISPER", nil, sender)
          return true
  		  end	
	    end
	  else
 		  local partyMemberName = UnitName("party"..i, false)
   	  if partyMemberName ~= nil then
        if not partyMemberName:find("-" .. GetRealmName())then
     	    partyMemberName = partyMemberName.."-"..GetRealmName()
        end
 	  	  if partyMemberName == sender then
  	  	  SendChatMessage(L["IsinParty"], "WHISPER", nil, sender)
          return true
		    end	
		  end
	  end
  end
  return false
end

local function is_in_instance()
  local instance, instanceType = IsInInstance()
  if instance ~= nil and instanceType == "party" and _G.AWI_DBPC.InstanceCheck then
	  return true
	end
	return false
end

local function add_party_or_raid(sender)
  if _G.AWI_DBPC.Debug then 
    myprint(sender) 
  end
  if not CheckGroupMembers(sender) then
    if _G.AWI_DBPC.Debug then 
      myprint(sender.." is not in party or raid") 
    end
    if _G.AWI_DBPC.RaidConvert and not IsInRaid() then
      if UnitInParty("party1") and GetNumSubgroupMembers() == 4 then
        if isRetail then
          if not is_in_instance() then
            C_PartyInfo.ConvertToRaid()
            if _G.AWI_DBPC.RaidInviteDelay then
              C_Timer.After(_G.AWI_DBPC.RaidInviteDelay, function() C_PartyInfo.InviteUnit(sender) end )
            else
              C_Timer.After(1, function() C_PartyInfo.InviteUnit(sender) end )
            end
            return
          end
        else
          if not is_in_instance() then
            ConvertToRaid()
            if _G.AWI_DBPC.RaidInviteDelay then
              C_Timer.After( _G.AWI_DBPC.RaidInviteDelay, function() InviteUnit(sender) end )
            else
              C_Timer.After(1, function() InviteUnit(sender) end )
            end
            InviteUnit(sender)
            return
          end
        end
      end
    end
    if UnitInParty("party1") and GetNumSubgroupMembers() == 4 and not IsInRaid() then
      SendChatMessage(L["FullParty"], "WHISPER", nil, sender)
      return    
    end
    if GetNumGroupMembers() == 40 and IsInRaid() then
      SendChatMessage(L["FullRaid"], "WHISPER", nil, sender)
      return
    end
    if isRetail then
      C_PartyInfo.InviteUnit(sender)
    else
      InviteUnit(sender)
    end
  end
end

local function getBNCharacterInfo(bnetIDAccount)
	local friendIndex = BNGetFriendIndex(bnetIDAccount) -- Check if they are on more than one client at once
	if not friendIndex then return end
	for loop = 1, BNGetNumFriendGameAccounts(friendIndex) do
		local _, charName, client, realmName, realmId, faction, _, _, _, _, _, _, _, _, _, bnetIDGameAccount = BNGetFriendGameAccountInfo(friendIndex, loop)
    if _G.AWI_DBPC.Debug then 
      myprint("sender name: "..charName)
      myprint("your client: ".._G.BNET_CLIENT_WOW)
	    myprint("sender client: "..client)
      myprint("sender faction: "..faction)
      myprint("sender realmname: "..realmName)
    end
    if client == _G.BNET_CLIENT_WOW and realmId > 0 then
		  return charName, realmName, bnetIDGameAccount
  	end
	end
end

-- https://wow.gamepedia.com/CHAT_MSG_BN_WHISPER
local function AWIBN_filter(self, event, msg, _, _, _, _, _, _, _, _, _, lineID, guid, bnetIDAccount, isMobile)
  
  if not _G.AWI_DBPC.BNet then return end
  
  if _G.AWI_DBPC.AutoInvite and isKeyinText(msg) and (not isMobile) and bnetIDAccount > 0 then
    if AWI_lineid ~= lineID then -- run once per lineid, each chat tab will trigger it again, same lineid used.
      AWI_lineid = lineID
      local author = ""
      local realm = ""
      if isRetail then
        if C_BattleNet then
          if _G.AWI_DBPC.Debug then 
            myprint(msg)
            myprint(bnetIDAccount) 
          end
          local accountInfo = C_BattleNet.GetAccountInfoByID(bnetIDAccount)
          if accountInfo.gameAccountInfo then 
            if accountInfo.gameAccountInfo.clientProgram then
              if _G.AWI_DBPC.Debug then 
                myprint(accountInfo.gameAccountInfo.clientProgram) 
              end
              if accountInfo.gameAccountInfo.clientProgram ~= "WoW" then return end
            end
            if accountInfo.gameAccountInfo.characterName then
              author = accountInfo.gameAccountInfo.characterName
            end
            if accountInfo.gameAccountInfo.realmName then
              realm = accountInfo.gameAccountInfo.realmName
            end          
          end
        end
        if _G.AWI_DBPC.Debug then 
          myprint(author) 
        end
        if author == "" then return end
        if isAuthorized(author, true) then
          if _G.AWI_DBPC.Debug then 
            myprint(author)
            myprint(realm)
          end
         if realm == "" then
            add_party_or_raid(author)
          else
            add_party_or_raid(author.."-"..realm)
          end
          if _G.AWI_DBPC.HideWhispers then
            return true
          end
        end
      else
        -- CE and BCC
        sender, realm, id = getBNCharacterInfo(bnetIDAccount)
        if not id then return end
        author = Ambiguate(sender, "none")
        if _G.AWI_DBPC.Debug then 
          myprint(author)
          myprint(realm)
        end
        if author == "" then return end
        if isAuthorized(author, true) then
          if realm == "" then
            add_party_or_raid(author)
          else
            add_party_or_raid(author.."-"..realm)
          end
          if _G.AWI_DBPC.HideWhispers then
            return true
          end
        end
	    end
	  elseif _G.AWI_DBPC.HideWhispers then
      return true
	  end
  end
end

-- https://wow.gamepedia.com/CHAT_MSG_WHISPER
local function AWI_filter(self, event, msg, sender, _, _, _, _, _, _, _, _, lineID, guid, bnetIDAccount, isMobile)
  msg = strtrim(msg)
  msg = msg:lower()
  if _G.AWI_DBPC.Debug then 
    myprint(sender)
    myprint(lineID)
    myprint(AWI_lineid)  
    myprint(msg)
  end
  if _G.AWI_DBPC.AutoInvite and isKeyinText(msg) and isAuthorized(sender, false) then
    if AWI_lineid ~= lineID then  -- run once per lineid, each chat tab will trigger it again with same lineid.
      AWI_lineid = lineID
      if _G.AWI_DBPC.Debug then 
        myprint(msg) 
      end
      add_party_or_raid(sender)
      if _G.AWI_DBPC.HideWhispers then
        return true
      end
    elseif _G.AWI_DBPC.HideWhispers then
      return true
    end
  end
end

-- https://wow.gamepedia.com/CHAT_MSG_WHISPER_INFORM
local function AWI_Inform_filter(self, event, msg, sender, _, _, _, _, _, _, _, _, lineID, guid, bnetIDAccount, isMobile)
  if _G.AWI_DBPC.Debug then 
    myprint(lineID)
    myprint(AWI_lineid) 
  end
  if _G.AWI_DBPC.AutoInvite then
    if AWI_lineid ~= lineID then  -- run once per lineid, each chat tab will trigger it again with same lineid.
      AWI_lineid = lineID
      if _G.AWI_DBPC.Debug then 
        myprint(msg) 
      end
      if string.find(msg, L["IsinParty"]) or 
         string.find(msg, L["IsinRaid"]) or 
         string.find(msg, L["FullParty"]) or 
         string.find(msg, L["FullRaid"]) then 
        if _G.AWI_DBPC.HideWhispers then
          return true
        end
      end
    end
  end
end

local Options = CreateFrame("Frame", "AutoWhisperInviteUIOptions", InterfaceOptionsFramePanelContainer, _G.BackdropTemplateMixin and "BackdropTemplate" or nil)
Options.name = addon

InterfaceOptions_AddCategory(Options)

if Options then
  Mixin(Options, BackdropTemplateMixin)
end

Options:Hide();
Options:SetScript("OnShow", function(self)
  local offset = 0
  
  local function makeCheckbox(label, description, onClick)
  local check = CreateFrame("CheckButton", "AutoWhisperInvite_Checkbox_"..label, self, "InterfaceOptionsCheckButtonTemplate")
  check:SetScript("OnClick", function(self)
    	-- 856 = igMainMenuOptionCheckBoxOn
	   	-- 857 = igMainMenuOptionCheckBoxOff
    local click = self:GetChecked() and 856 or 857
  	PlaySound(click) onClick(self, self:GetChecked() and true or false)	end)
  	check.label = _G[check:GetName().."Text"]
  	check.label:SetWidth(800)
  	check.label:SetText(label)
  	check.tooltipText = label
  	check.tooltipRequirement = description
  	return check
  end
  
  -- Config Title
  local TitleOptions = self:CreateFontString("$parentTitleOptions", "ARTWORK", "GameFontNormalLarge")
  TitleOptions:SetPoint("TOPLEFT", 16, -16)
  TitleOptions:SetText(_G.AWI_DB.Color1..addon)

  -- version
  local VersionOptions = self:CreateFontString("$parentTitleOptions", "ARTWORK", "GameFontNormal");
  VersionOptions:SetPoint("TOPRIGHT", -16, -17)
  VersionOptions:SetJustifyH("RIGHT")
  VersionOptions:SetText(_G.AWI_DB.Color2..L["Version"].._G.AWI_DB.Version)

  
  -- Global Saves On/Off
  AutoWhisperInvite_Checkbox_GlobalSave = makeCheckbox(
  	L["BTNGS"] ,
  	L["BTNDGS"] ,	-- description
  	function(self, value)
		  _G.AWI_DB.GlobalSave  = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  local offset = - 20
  AutoWhisperInvite_Checkbox_GlobalSave:SetChecked(_G.AWI_DB.GlobalSave);
  AutoWhisperInvite_Checkbox_GlobalSave:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);
		
  -- Auto Invite On/Off
  AutoWhisperInvite_Checkbox_AutoInvite = makeCheckbox(
  	L["BTN1"] ,
  	L["BTND1"] ,	-- description
  	function(self, value)
	    _G.AWI_DBPC.AutoInvite = value
      LoadGlobal()
      if _G.AWI_DBPC.AutoInvite then
        ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
        ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
        ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
      else
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
      end
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_AutoInvite:SetChecked(_G.AWI_DBPC.AutoInvite);
  AutoWhisperInvite_Checkbox_AutoInvite:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Friends whisper invites
  AutoWhisperInvite_Checkbox_Friends = makeCheckbox(
  	L["BTN2"] ,
  	L["BTND2"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.FriendsInvite = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_Friends:SetChecked(_G.AWI_DBPC.FriendsInvite);
  AutoWhisperInvite_Checkbox_Friends:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- BNet whisper invites
  AutoWhisperInvite_Checkbox_BNet = makeCheckbox(
  	L["BTN3"] ,
  	L["BTND3"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.BNet = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_BNet:SetChecked(_G.AWI_DBPC.BNet);
  AutoWhisperInvite_Checkbox_BNet:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- All whisper invites
  AutoWhisperInvite_Checkbox_All = makeCheckbox(
  	L["BTN4"] ,
  	L["BTND4"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.AllInvite = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_All:SetChecked(_G.AWI_DBPC.AllInvite);
  AutoWhisperInvite_Checkbox_All:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Convert to raid On/Off
  AutoWhisperInvite_Checkbox_RaidConvert = makeCheckbox(
  	L["BTN5"] ,
  	L["BTND5"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.RaidConvert = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_RaidConvert:SetChecked(_G.AWI_DBPC.RaidConvert);
  AutoWhisperInvite_Checkbox_RaidConvert:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Don't convert to raid if in instance
  AutoWhisperInvite_Checkbox_InstanceCheck = makeCheckbox(
  	L["BTN6"] ,
  	L["BTND6"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.InstanceCheck = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_InstanceCheck:SetChecked(_G.AWI_DBPC.InstanceCheck);
  AutoWhisperInvite_Checkbox_InstanceCheck:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Exact Keyword in Whisper On/Off
  AutoWhisperInvite_Checkbox_ExactWhispers = makeCheckbox(
  	L["BTN7"] ,
  	L["BTND7"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.ExactWhispers = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_ExactWhispers:SetChecked(_G.AWI_DBPC.ExactWhispers);
  AutoWhisperInvite_Checkbox_ExactWhispers:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Hide Whisper On/Off
  AutoWhisperInvite_Checkbox_HideWhispers = makeCheckbox(
  	L["BTN8"] ,
  	L["BTND8"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.HideWhispers = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_HideWhispers:SetChecked(_G.AWI_DBPC.HideWhispers);
  AutoWhisperInvite_Checkbox_HideWhispers:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Show Settings On/Off
  AutoWhisperInvite_Checkbox_Status = makeCheckbox(
  	L["BTN9"] ,
  	L["BTND9"] ,	-- description
  	function(self, value)
		  _G.AWI_DBPC.Status = value
      LoadGlobal()
  	end);	-- onClick functions or commands 
  offset = offset - 20
  AutoWhisperInvite_Checkbox_Status:SetChecked(_G.AWI_DBPC.Status);
  AutoWhisperInvite_Checkbox_Status:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);

  -- Panel Info

  local function MakePanelInfo(ftext, foffset)
    offset = offset - foffset
    local PanelInfo = self:CreateFontString("$parentTitleOptions", "ARTWORK", "GameFontNormal")
    PanelInfo:SetJustifyH("LEFT")
    PanelInfo:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", 0, offset);
    PanelInfo:SetWidth(800)
    PanelInfo:SetText(_G.AWI_DB.Color2..ftext)
  end

  local function makeEditbox(fpoffset, foffset, fwidth, fheight, ftext, fname)
    if foffset ~= 0 then offset = offset - foffset end
    local editb = CreateFrame("EditBox", fname, self, "OptionsButtonTemplate", _G.BackdropTemplateMixin and "BackdropTemplate" or nil)
    editb:SetWidth(fwidth)
    editb:SetHeight(fheight)
    editb:SetFontObject(GameFontNormalLarge)
    Mixin(editb, BackdropTemplateMixin)
    editb:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
  		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = true, tileSize = 16, edgeSize = 16,
			insets = { left = 4, right = 4, top = 4, bottom = 4 }})
    editb:SetBackdropColor(0.1,0.1,0.1,0.5)
    editb:SetBackdropBorderColor(0.6, 0.6, 0.6, 1)    
    editb:SetPoint("TOPLEFT", TitleOptions, "BOTTOMLEFT", fpoffset, offset);
    editb:ClearFocus(self)
    editb:SetAutoFocus(false)
    editb:SetTextInsets(10,0,0,0)
    editb:SetText(ftext)
    return editb
  end

  local function makeEditboxButton(fpoffset, foffset, fwidth, fheight, ftext, fname)
    if foffset ~= 0 then offset = offset - foffset end
    local editb = CreateFrame("Button", nil, fname, "OptionsButtonTemplate")
    editb:Enable(true)
    editb:SetWidth(fwidth)
    editb:SetHeight(fheight)
    editb:SetText(ftext)
    editb:SetPoint("TOPLEFT", TitleOptions ,"BOTTOMLEFT", fpoffset, offset)
    return editb
  end

  MakePanelInfo(L["RaidInviteSetting"], 40)

  -- Show Raid Invite Delay
  AutoWhisperInvite_Editbox_ShowRID = makeEditbox(300, -10, 60, 30, _G.AWI_DBPC.RaidInviteDelay, "AutoWhisperInvite_EditBox_RID")
  AutoWhisperInvite_Editbox_ShowRID:SetScript("OnEnterPressed",function(self) self:ClearFocus()
    local trdi = AutoWhisperInvite_Editbox_ShowRID:GetText()
    local rdi = tonumber(trdi)
    if rdi == nil then
      myprint(string.format(L["RaidInviteDelayInvalid"].." not "..trdi.._G.AWI_DB.Color2))
      _G.AWI_DBPC.RaidInviteDelay = 1
      LoadGlobal()
      AutoWhisperInvite_Editbox_ShowRID:SetText("")
      AutoWhisperInvite_Editbox_ShowRID:SetText(_G.AWI_DBPC.RaidInviteDelay)
      myprint(L["RaidInviteSetting"].." ".._G.AWI_DBPC.RaidInviteDelay)
    elseif rdi < 0.1 or rdi > 2 then
      myprint(string.format(L["RaidInviteDelayInvalid"].." not "..rdi.._G.AWI_DB.Color2))
      _G.AWI_DBPC.RaidInviteDelay = 1
      LoadGlobal()
      AutoWhisperInvite_Editbox_ShowRID:SetText("")
      AutoWhisperInvite_Editbox_ShowRID:SetText(_G.AWI_DBPC.RaidInviteDelay)
      myprint(L["RaidInviteSetting"].." ".._G.AWI_DBPC.RaidInviteDelay)
    else
      _G.AWI_DBPC.RaidInviteDelay = rdi
      LoadGlobal()
      AutoWhisperInvite_Editbox_ShowRID:SetText("")
      AutoWhisperInvite_Editbox_ShowRID:SetText(_G.AWI_DBPC.RaidInviteDelay)
      myprint(L["RaidInviteSetting"].." ".._G.AWI_DBPC.RaidInviteDelay)
    end
  end)
  AutoWhisperInvite_Editbox_ShowRID:SetScript("OnEscapePressed",function(self) self:ClearFocus()
    AutoWhisperInvite_Editbox_ShowRID:SetText("")
    AutoWhisperInvite_Editbox_ShowRID:SetText(_G.AWI_DBPC.RaidInviteDelay)
  end)
 
  MakePanelInfo(L["ACPInfo1"], 40)
  MakePanelInfo(L["ACPInfo2"], 20)
  MakePanelInfo(L["ACPInfo3"], 20)
  MakePanelInfo(L["ACPInfo4"], 20)
  MakePanelInfo(L["ACPInfo5"], 20)
  MakePanelInfo(L["ACPInfo6"], 20)

  -- Show Keywords
  AutoWhisperInvite_Editbox_ShowKeys = makeEditbox(0, 20, 600, 30, "|cffffffff"..getKeys(), "AutoWhisperInvite_EditBox_Keys")
  AutoWhisperInvite_Editbox_ShowKeys:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
  AutoWhisperInvite_Editbox_ShowKeys:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)

      
  -- Add/Remove keywords
  AutoWhisperInvite_Editbox_Keywords = makeEditbox(0, 55, 130, 30, "", "AutoWhisperInvite_EditBox_Keys")
  AutoWhisperInvite_Editbox_Keywords:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
  AutoWhisperInvite_Editbox_Keywords:SetScript("OnEscapePressed", function(self) self:ClearFocus() end)
   
  -- add keyword button
  AutoWhisperInvite_Editbox_Keywords.add = makeEditboxButton(140, 0, 130, 30, "Add keyword", AutoWhisperInvite_Editbox_Keywords)
  AutoWhisperInvite_Editbox_Keywords.add:SetScript("OnClick",function() 
    addKeyword(AutoWhisperInvite_Editbox_Keywords:GetText())
    AutoWhisperInvite_Editbox_Keywords:SetText("")
    AutoWhisperInvite_Editbox_ShowKeys:SetText("|cffffffff"..getKeys())   
  end)

  -- remove keyword button
  AutoWhisperInvite_Editbox_Keywords.del = makeEditboxButton(280, 0, 140, 30, "Remove keyword", AutoWhisperInvite_Editbox_Keywords)
  AutoWhisperInvite_Editbox_Keywords.del:SetScript("OnClick",function() 
    delKeyword(AutoWhisperInvite_Editbox_Keywords:GetText()) 
    AutoWhisperInvite_Editbox_Keywords:SetText("")
    AutoWhisperInvite_Editbox_ShowKeys:SetText("|cffffffff"..getKeys())   
  end)
  
  self:SetScript("OnShow", nil);
end)

-- RegisterEvents
local AWI_Eventframe = CreateFrame("FRAME")
AWI_Eventframe:RegisterEvent("ADDON_LOADED")
AWI_Eventframe:RegisterEvent("PLAYER_LOGIN")

local function AWI_OnEvent(self, event, arg1, arg2, ...)
  if event == "ADDON_LOADED" and arg1 == addon then
    myprint(L["Addon_Loaded"])
  	AWI_Loaded = true
    AWI_Eventframe:UnregisterEvent("ADDON_LOADED")
  end
  if event == "PLAYER_LOGIN" and AWI_Loaded then
    myprint(L["Player_Loaded"])
    _G.AWI_DB.Color1 = color1
    _G.AWI_DB.Color2 = color2
    _G.AWI_DB.Version = GetAddOnMetadata(addon, "X-Version")
 
    -- Initialize if new variable added

    if _G.AWI_DB.GlobalSave == nil then
      _G.AWI_DB.GlobalSave = false
    end    
    if _G.AWI_DBPC.AllInvite == nil then
      _G.AWI_DBPC.AllInvite = false
    end
    if _G.AWI_DBPC.AutoInvite == nil then
      _G.AWI_DBPC.AutoInvite = false
    end
    if _G.AWI_DBPC.BNet == nil then
      _G.AWI_DBPC.BNet = false
    end
    if _G.AWI_DBPC.Debug == nil then
      _G.AWI_DBPC.Debug = false
    end
    if _G.AWI_DBPC.ExactWhispers == nil then
      _G.AWI_DBPC.ExactWhispers = true
    end
    if _G.AWI_DBPC.FriendsInvite == nil then
      _G.AWI_DBPC.FriendsInvite = true
    end
    if _G.AWI_DBPC.HideWhispers == nil then
      _G.AWI_DBPC.HideWhispers = false
    end                    
    if _G.AWI_DBPC.RaidConvert == nil then
      _G.AWI_DBPC.RaidConvert = false
    end
    if _G.AWI_DBPC.RaidInviteDelay == nil then
      _G.AWI_DBPC.RaidInviteDelay = 1
    end  
    if tonumber(_G.AWI_DBPC.RaidInviteDelay) == nil then
      _G.AWI_DBPC.RaidInviteDelay = 1
    end      
    if _G.AWI_DBPC.Status == nil then
      _G.AWI_DBPC.Status = true
    end 
    if _G.AWI_DBPC.AutoInvite then
      ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
      ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
      ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
    else
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
    end

    if _G.AWI_DB.GlobalSave then
      LoadLocal()
    end
    
    LoadGlobal()

    if _G.AWI_DBPC.Status then   

      if _G.AWI_DB.GlobalSave then
        myprint(L["GlobalON"])
      else
        myprint(L["GlobalOFF"])
      end
      if _G.AWI_DBPC.AllInvite then
        myprint(L["AllON"])
      else
        myprint(L["AllOFF"])
      end
      if _G.AWI_DBPC.AutoInvite then
        myprint(L["AutoON"])
      else
		    myprint(L["AutoOFF"])
      end
      if _G.AWI_DBPC.ExactWhispers then
        myprint(L["ExactON"])
      else
        myprint(L["ExactOFF"])
      end
      if _G.AWI_DBPC.BNet then
        myprint(L["BNetON"])
      else
        myprint(L["BNetOFF"])
      end      
      if _G.AWI_DBPC.FriendsInvite then
        myprint(L["FriendsON"])
      else
        myprint(L["FriendsOFF"])
      end
      if _G.AWI_DBPC.HideWhispers then
        myprint(L["WhispersHidden"])
      else
        myprint(L["WhispersShown"])
      end
      if _G.AWI_DBPC.InstanceCheck then
        myprint(L["IsInInsanceOn"])
      else
        myprint(L["IsInInsanceOff"])
      end
      if _G.AWI_DBPC.RaidConvert then
        myprint(L["RaidConvertOn"])
      else
        myprint(L["RaidConvertOff"])
      end
      if _G.AWI_DBPC.RaidInviteDelay then
        myprint(L["RaidInviteSetting"].._G.AWI_DBPC.RaidInviteDelay)
      end   
      if _G.AWI_DBPC.Status then
        myprint(L["STATUSON"])
      else
        myprint(L["STATUSOFF"])
      end      
      print("...")
    end
    AWI_Eventframe:UnregisterEvent("PLAYER_LOGIN")
  end
end
AWI_Eventframe:SetScript("OnEvent", AWI_OnEvent)

_G.SLASH_AWI1  = "/awi"
_G.SLASH_AWI2  = "/awi "..L["SLAdd"]
_G.SLASH_AWI3  = "/awi "..L["SLAll"]
_G.SLASH_AWI4  = "/awi "..L["SLAuto"]
_G.SLASH_AWI5  = "/awi "..L["SLBNet"]
_G.SLASH_AWI6  = "/awi "..L["SLConfig"]
_G.SLASH_AWI7  = "/awi "..L["SLDebug"]
_G.SLASH_AWI8  = "/awi "..L["SLExact"]
_G.SLASH_AWI9 = "/awi "..L["SLFriends"]
_G.SLASH_AWI10 = "/awi "..L["SLGlobal"]
_G.SLASH_AWI11 = "/awi "..L["SLHelp"]
_G.SLASH_AWI12 = "/awi "..L["SLHide"]
_G.SLASH_AWI13 = "/awi "..L["SLList"]
_G.SLASH_AWI14 = "/awi "..L["SLInstance"]
_G.SLASH_AWI15 = "/awi "..L["SLRaid"]
_G.SLASH_AWI16 = "/awi "..L["SLRemove"]
_G.SLASH_AWI17 = "/awi "..L["SLStatus"]
_G.SLASH_AWI18 = "/autowhisperinvite"
_G.SlashCmdList.AWI = function(msg)
  msg = msg:lower()
  local sl_cmd, arg1, arg2, arg3 = strsplit(" ", msg)
  local showhelp = true
  if sl_cmd == L["SLDebug"] then
    showhelp = false
    _G.AWI_DBPC.Debug = not _G.AWI_DBPC.Debug
    if _G.AWI_DBPC.Debug then
      myprint(L["DebugON"])
    else
      myprint(L["DebugOFF"])
    end
  end
  if sl_cmd == L["SLAdd"] and arg1 ~= "" then
    showhelp = false
    addKeyword(arg1)
    if AutoWhisperInvite_Editbox_ShowKeys ~= nil then
      AutoWhisperInvite_Editbox_ShowKeys:SetText("|cffffffff"..getKeys())
    end
  end
  if sl_cmd == L["SLAll"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.AllInvite = not _G.AWI_DBPC.AllInvite
    if AutoWhisperInvite_Checkbox_All ~= nil then
  		AutoWhisperInvite_Checkbox_All:SetChecked(_G.AWI_DBPC.AllInvite);
  	end
  end
  if sl_cmd == L["SLBNet"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.BNet = not _G.AWI_DBPC.BNet
    if AutoWhisperInvite_Checkbox_BNet ~= nil then
  		AutoWhisperInvite_Checkbox_BNet:SetChecked(_G.AWI_DBPC.BNet);
  	end
  end
  if sl_cmd == L["SLAuto"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.AutoInvite = not _G.AWI_DBPC.AutoInvite
    if AutoWhisperInvite_Checkbox_AutoInvite ~= nil then
  		AutoWhisperInvite_Checkbox_AutoInvite:SetChecked(_G.AWI_DBPC.AutoInvite);
  	end
    if _G.AWI_DBPC.AutoInvite then
      ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
      ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
      ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
    else
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", AWI_filter)
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", AWI_Inform_filter)
      ChatFrame_RemoveMessageEventFilter("CHAT_MSG_BN_WHISPER", AWIBN_filter)
    end
  end
  if sl_cmd == L["SLExact"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.ExactWhispers = not _G.AWI_DBPC.ExactWhispers
    if AutoWhisperInvite_Checkbox_ExactWhispers ~= nil then
  		AutoWhisperInvite_Checkbox_ExactWhispers:SetChecked(_G.AWI_DBPC.ExactWhispers);
  	end
  end  
  if sl_cmd == L["SLFriends"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.FriendsInvite = not _G.AWI_DBPC.FriendsInvite
    if AutoWhisperInvite_Checkbox_Friends ~= nil then
  		AutoWhisperInvite_Checkbox_Friends:SetChecked(_G.AWI_DBPC.FriendsInvite);
  	end
  end
  if sl_cmd == L["SLGlobal"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DB.GlobalSave = not _G.AWI_DB.GlobalSave
    if AutoWhisperInvite_Checkbox_GlobalSave ~= nil then
  		AutoWhisperInvite_Checkbox_GlobalSave:SetChecked(_G.AWI_DB.GlobalSave);
  	end
  end
  if sl_cmd == L["SLHide"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.HideWhispers = not _G.AWI_DBPC.HideWhispers
    if AutoWhisperInvite_Checkbox_HideWhispers ~= nil then
  		AutoWhisperInvite_Checkbox_HideWhispers:SetChecked(_G.AWI_DBPC.HideWhispers);
  	end
  end
  if sl_cmd == L["SLList"] then
    showhelp = false
    myprint(string.format(L["Keys"],_G.AWI_DB.Color1..getKeys().._G.AWI_DB.Color2))
  end  
  if sl_cmd == L["SLInstance"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.InstanceCheck = not _G.AWI_DBPC.InstanceCheck
    if AutoWhisperInvite_Checkbox_InstanceCheck ~= nil then
  		AutoWhisperInvite_Checkbox_InstanceCheck:SetChecked(_G.AWI_DBPC.InstanceCheck);
  	end
  end
  if sl_cmd == L["SLRaid"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.RaidConvert = not _G.AWI_DBPC.RaidConvert
    if AutoWhisperInvite_Checkbox_RaidConvert ~= nil then
  		AutoWhisperInvite_Checkbox_RaidConvert:SetChecked(_G.AWI_DBPC.RaidConvert);
  	end
  end
  if sl_cmd == L["SLRemove"] and arg1 ~= "" then
    showhelp = false
    delKeyword(arg1)
    if AutoWhisperInvite_Editbox_ShowKeys ~= nil then
      AutoWhisperInvite_Editbox_ShowKeys:SetText("|cffffffff"..getKeys())
    end
  end
  if sl_cmd == L["SLStatus"] then
    showhelp = false
    sl_cmd = "help"
    _G.AWI_DBPC.Status = not _G.AWI_DBPC.Status
    if AutoWhisperInvite_Checkbox_Status ~= nil then
  		AutoWhisperInvite_Checkbox_Status:SetChecked(_G.AWI_DBPC.Status);
  	end
  end
	if sl_cmd == L["SLConfig"] then
		InterfaceOptionsFrame_OpenToCategory(addon); -- get over options bug...
		InterfaceOptionsFrame_OpenToCategory(addon); -- workaround to get to options call twice!
	end
	
  if _G.AWI_DB.GlobalSave then
    LoadGlobal()
  end
	
	if sl_cmd == "" or sl_cmd == "autowhisperinvite" then
    showhelp = false
	end
  if sl_cmd == "" or sl_cmd == "autowhisperinvite" or sl_cmd == L["SLHelp"] then
    myprint(".")

    if _G.AWI_DB.GlobalSave then
      myprint(L["GlobalON"])
    else
      myprint(L["GlobalOFF"])
    end

    if _G.AWI_DBPC.AllInvite then
      myprint(L["AllON"])
    else
      myprint(L["AllOFF"])
    end

    if _G.AWI_DBPC.AutoInvite then
      myprint(L["AutoON"])
    else
      myprint(L["AutoOFF"])
    end

    if _G.AWI_DBPC.BNet then
      myprint(L["BNetON"])
    else
      myprint(L["BNetOFF"])
    end    

    if _G.AWI_DBPC.FriendsInvite then
      myprint(L["FriendsON"])
    else
      myprint(L["FriendsOFF"])
    end 

    if _G.AWI_DBPC.ExactWhispers then
      myprint(L["ExactON"])
    else
      myprint(L["ExactOFF"])
    end  

    if _G.AWI_DBPC.HideWhispers then
      myprint(L["WhispersHidden"])
    else
      myprint(L["WhispersShown"])
    end    

    if _G.AWI_DBPC.InstanceCheck then
      myprint(L["IsInInsanceOn"])
    else
      myprint(L["IsInInsanceOff"])
    end

    if _G.AWI_DBPC.RaidConvert then
      myprint(L["RaidConvertOn"])
    else
      myprint(L["RaidConvertOff"])
    end
    
    if _G.AWI_DBPC.RaidInviteDelay then
      myprint(L["RaidInviteSetting"].._G.AWI_DBPC.RaidInviteDelay)
    end    

    if _G.AWI_DBPC.Status then
      myprint(L["STATUSON"])
    else
      myprint(L["STATUSOFF"])
    end

    if showhelp then
      myprint(L["Help"])
    end
  end
end
